;==============================================================================
;
; Program:         Landbank.au3
; Description:     Allow Users to access the data Import and Test facilities.
; Parameters:      None - 
; Requirement(s):  A SQLite3 database with SQLite3.dll and AdoDB for MS-Excel.
; Return Value(s): N/A
; Author(s):       DJ van Vliet
; Note(s):         The SQLite.dll file is included in the .exe file.
;==============================================================================

Global Const $gSQLiteDatabase = @ScriptDir & "\" & "Landbank.db"
Global Const $WM_NOTIFY = 0x004E
Global Const $DebugIt = 1
 
;ListView Events
Global Const $NM_FIRST = 0
Global Const $NM_LAST = (-99)
Global Const $NM_OUTOFMEMORY = ($NM_FIRST - 1)
Global Const $NM_CLICK = ($NM_FIRST - 2)
Global Const $NM_DBLCLK = ($NM_FIRST - 3)
;~ Global Const $NM_RETURN = ($NM_FIRST - 4)
;~ Global Const $NM_RCLICK = ($NM_FIRST - 5)
;~ Global Const $NM_RDBLCLK = ($NM_FIRST - 6)
;~ Global Const $NM_SETFOCUS = ($NM_FIRST - 7)
;~ Global Const $NM_KILLFOCUS = ($NM_FIRST - 8)
;~ Global Const $NM_CUSTOMDRAW = ($NM_FIRST - 12)
;~ Global Const $NM_HOVER = ($NM_FIRST - 13)
;~ Global Const $NM_NCHITTEST = ($NM_FIRST - 14)
;~ Global Const $NM_KEYDOWN = ($NM_FIRST - 15)
;~ Global Const $NM_RELEASEDCAPTURE = ($NM_FIRST - 16)
;~ Global Const $NM_SETCURSOR = ($NM_FIRST - 17)
;~ Global Const $NM_CHAR = ($NM_FIRST - 18)
;~ Global Const $NM_TOOLTIPSCREATED = ($NM_FIRST - 19)

Global $msg	; application wide message variable
Global $aCSV 						;Array of imported file lines
Global $aField					;Array of Line split by semi-colon
Global $flPath					;Path of the file to be imported

; Global arrays for holding Form controls
Global $afrmLandbank, $afrmImport, $afrmTestTx, $afrmRelease

;Test
Global $ListView

;Global $afrmLandbank, $afrmLogin, $afrmSelectDatabase
;Global $afrmRaised, $afrmRaisedExport, $afrmRaisedSummary
;Global $afrmPayments, $afrmPaymentImport, $afrmPaymentSummary 
;Global $afrmStats, $afrmStatExport,$afrmStatSummary
;Global $afrmMNTNServers, $afrmServerAdd, $afrmServerMod
;Global $afrmMNTNDatabases, $afrmDatabaseAdd, $afrmDatabaseMod
;Global $afrmMNTNUsers, $afrmUserAdd, $afrmUserMod

;Global User variables
Global $hSQLite
Global $gServer = "TEBSQLTMP", $gDatabase = "TEBSchemes"
Global $gUserID = "", $gRights = "   "

; AutoIt Options
Opt("MustDeclareVars", 1)	; Variables must be pre-declared
Opt("TrayIconHide", 1)	; Default tray menu items (Script Paused/Exit) will not be shown.

; Include project files
#include <GUIConstants.au3>
#include <c:\AutoIt3\Projects\TEB\Include\SQLiteDjvv.au3>
#include <GuiStatusBar.au3>
#include <GuiListView.au3>
#include <math.au3>
#include "frmLandbank.au3"
#include "frmImport.au3"
#include "frmTestTx.au3"
#include "frmRelease.au3"

;Load SQLite3.dll
_SQLite_Startup("..\Dlls\SQLite3.dll")
If @error = 1 Then
	MsgBox(0, "SQLite","Could not load SQLite3.dll - aborting ...")
	exit
EndIf

If FileExists($gSQLiteDatabase) Then
	$hSQLite = _SQLite_Open ($gSQLiteDatabase)
	If $hSQLite = 0 Then ;Error Landbank.db not found
		MsgBox(0, "SQLite","Could not open database: " & $gSQLiteDatabase & " - aborting ...")
		_SQLite_Shutdown()		;Unload SQLite3.dll
		exit
	EndIf
Else
		MsgBox(0, "SQLite","Database: " & $gSQLiteDatabase & " not found - aborting ...")
		_SQLite_Shutdown()		;Unload SQLite3.dll	
		exit
EndIf


CreateForm_frmLandbank($afrmLandbank) ; frmLandbank is main window
CreateForm_frmImport($afrmImport, $afrmLandbank[0][1]) ; frmImport is child window to frmLandbank
CreateForm_frmTestTx($afrmTestTx, $afrmLandbank[0][1]) ; frmTestTx is child window to frmLandbank
CreateForm_frmRelease($afrmRelease, $afrmTestTx[0][1]) ; frmRelease is a child window to frmTestTX

GUISetState(@SW_SHOW, $afrmLandbank[0][1])

frmLandbank_Initialize()

;Register WM_NOTIFY  events
;;;;;;;;GUIRegisterMsg($WM_NOTIFY, "WM_Notify_Events")

While 1
	$msg = GuiGetMsg(1)
	
	; Prevent unnessary calls to _GetCtrl() by excluding $GUI_EVENT_MOUSEMOVE
	If $msg[0] <> $GUI_EVENT_MOUSEMOVE Then
		Select
		Case $msg[1] = $afrmLandbank[0][1]
			frmLandbank_WndProc()
		Case $msg[1] = $afrmImport[0][1]
			frmImport_WndProc()
 		Case $msg[1] = $afrmTestTx[0][1]
 			frmTestTx_WndProc()
 		Case $msg[1] = $afrmRelease[0][1]
 			frmRelease_WndProc()
;~ 		Case $msg[1] = $afrmMNTNServers[0][1]
;~ 			frmMNTNServers_WndProc()
			
		EndSelect
	EndIf
WEnd


Func Landbank_WndProc()
	Select
		
	Case $msg[0] = $GUI_EVENT_CLOSE
		GUIDelete($afrmLandbank[0][1])
		GUIDelete($afrmImport[0][1])
		GUIDelete($afrmTestTx[0][1])

		_SQLite_Close($hSQLite)		;Close TEBSchemes.db
		_SQLite_Shutdown()		;Unload SQLite3.dll
		
	Exit
	
	EndSelect
EndFunc

Func frmRelease_LV_Click()
    ;----------------------------------------------------------------------------------------------
		ConsoleWrite("Single Click" &  _GUICtrlListViewGetItemText (_GetCtrl("$lvClients", $afrmRelease), _
			                           _GUICtrlListViewGetSelectedIndices(_GetCtrl("$lvClients", $afrmRelease))) & @LF)
		;----------------------------------------------------------------------------------------------
EndFunc   ;==>ListView_Click
 
Func frmRelease_LV_DoubleClick()
     ;----------------------------------------------------------------------------------------------
     MsgBox(0,"Double Clicked", _GUICtrlListViewGetItemText (_GetCtrl("$lvClients", $afrmRelease), _
			                           _GUICtrlListViewGetSelectedIndices(_GetCtrl("$lvClients", $afrmRelease))))
     ;----------------------------------------------------------------------------------------------
;     MsgBox(0,"Double Clicked", _GUICtrlListViewGetItemText ($ListView, _GUICtrlListViewGetSelectedIndices($ListView)))
EndFunc   ;==>ListView_DoubleClick

Func _DebugHeader($s_text)
     Return _
               "!===========================================================" & @LF & _
               "+===========================================================" & @LF & _
               "-->" & $s_text & @LF & _
               "+===========================================================" & @LF
EndFunc   ;==>_DebugHeader  
;
; WM_NOTIFY event handler
Func WM_Notify_Events($hWndGUI, $MsgID, $wParam, $lParam)
     #forceref $hWndGUI, $MsgID, $wParam
     Local $tagNMHDR, $event, $hwndFrom, $code
     $tagNMHDR = DllStructCreate("int;int;int", $lParam) ;NMHDR (hwndFrom, idFrom, code)
     If @error Then Return
     $event = DllStructGetData($tagNMHDR, 3)
     Select
     Case $wParam = _GetCtrl("$lvClients", $afrmRelease)
          Select
               Case $event = $NM_CLICK
                    frmRelease_LV_Click ()
               Case $event = $NM_DBLCLK
                    frmRelease_LV_DoubleClick ()
               EndSelect
     Case $wParam = _GetCtrl("$tb", $afrmRelease)
          Select
               Case $event = $NM_CLICK
                    frmRelease_LV_Click ()
               Case $event = $NM_DBLCLK
                    frmRelease_LV_DoubleClick ()
               EndSelect
     EndSelect
     $tagNMHDR = 0
     $event = 0
     $lParam = 0
EndFunc   ;==>WM_Notify_Events

Func _GetCtrl($sCtrlName, $afrmArray)    
	For $i = 0 to UBound($afrmArray, 1) - 1
        If $afrmArray[$i][0] = $sCtrlName Then
            Return $afrmArray[$i][1]
        EndIf
    Next
    
	; Error Handler
    If $i > UBound($afrmArray, 1) - 1 Then
        Main_ErrorHandler("Control reference invalid ", _
                                "Control Name: " & $sCtrlName  & @CRLF & _
                                "Form Name: " & $afrmArray[0][0], "Fatal")
        SetError(1)
    EndIf
EndFunc

Func FixDate(ByRef $sDate)
	Local $aDate, $aDayTime, $theDate
	$aDate = StringSplit($sDate, "/")
	$aDayTime = StringSplit($aDate[3], " ")
	$theDate = $aDate[1]
	if StringLen($aDate[2])=1 Then 
		$theDate &= "0" & $aDate[2] 
	Else 
		$theDate &= $aDate[2] 
	EndIf
	if StringMid ($aDate[3],2,1)=' ' Then 
		$theDate &= "0" & StringLeft($aDate[3],1) 
	Else 
		$theDate &= StringLeft($aDate[3],2) 
	EndIf
	$sDate = $theDate
EndFunc

Func AgeNextBirthDay($sDateFrom, $sDateTo)
	Local $iANB
	$iANB = StringLeft($sDateTo,4) - StringLeft($sDateFrom,4)
	If StringMid($sDateTo,5,2) < StringMid($sDateFrom,5,2) _
		Or (StringMid($sDateTo,5,2) = StringMid($sDateFrom,5,2) _
		   And StringMid($sDateTo,7,2) < StringMid($sDateFrom,7,2)) Then
			$iANB -= 1
	EndIf
	;$iDateCalc contains age at $sDateTo	
	$iANB += 1	; Add 1 for age next Birthday

;MsgBox(0,"ANB", "Age Next Birthday: " & $iDateCalc)
	Return $iANB
EndFunc

; Application wide central message handler.
; Displays an error msg and exits the application if the error is fatal.
Func Main_ErrorHandler($sError, $sSolution, $sImpact)
    If $sImpact = "Non-Fatal" Then
        MsgBox(8256, "User Manager", $sError & @CRLF & @CRLF & $sSolution)
    ElseIf $sImpact = "Fatal" Then    
        MsgBox(8208, "User Manager", $sError & @CRLF & @CRLF & $sSolution)
        Exit
    EndIf
EndFunc

;===============================================================================
;
; Description:      Converts a currency string from
;                             1000000.00 to 1,000,000.00
;                             $1000000.00 to $1,000,000.00
;                             1000000.00$ to 1,000,000.00$
;
; Parameter(s):     $sCurrency		-	String containing the Currency Amount
;                   $cDecimalChar	-	Decimal point character (default='.')
;                   $nWidth			-   Number of characters between delimiters (default=3)
;                   $cChar			-	Delimiter Character (default=',')
;                   $sHasCSymbol	-	Does sCurrency contains a currency Symbol (default=0)
;                                               0 - No Symbol
;                                               1 - Yes (prefix)
;                                               2 - No (suffix)
;                   $nCSymbolSize	-	Size of Currency Symbol (default=0)
;                                               0 - No Symbol
;                                               n - Sysmbol size (in chars)
; Requirement(s):   None
; Return Value(s):  On Success - Returns the formatted Currency String
;                   On Failure - 0  and sets @ERROR = 1
; Author(s):        Douglas van Vliet
; Note(s):          None
;
;===============================================================================
Func _StringCurrencyFormat($sCurrency, $cDecimalChar='.', $nWidth=3, $cChar=',', $sHasCSymbol=0, $nCSymbolSize=0)
	Local $sNewCurrency, $aCurrency[5], $sCurrValue, $sSymbol, $nLength, $nNoDel, $aStripCommas, $sStrippedCurrency
	Local $nPreDel, $sDelStr, $sDelValue, $nCnt
	
;MsgBox(4096,"$sCurrency",$sCurrency)

	$aCurrency = 0
	$aStripCommas = 0

	;Strip all $cChar from SUmAssured/Premium - previous calculation
	$aStripCommas = StringSplit($sCurrency, $cChar)
	$sStrippedCurrency = ""
	For $idx = 1 to $aStripCommas[0]
		$sStrippedCurrency &= $aStripCommas[$idx]
	Next

	; Verify If $sCurrency is valid - First Char is not a $cDecimalChar
	If (StringLeft($sCurrency,1) = $cDecimalChar) Then
		SetError(1)
		Return $sCurrency
	EndIf

	;Split the Value into Dollars and Cents
	$aCurrency = StringSplit($sStrippedCurrency, $cDecimalChar)
;MsgBox(4096,"$sCurrency", "$sStrippedCurrency" & $sStrippedCurrency & " - " & $aCurrency[0] & " - $aCurrency[1]" & $aCurrency[1])
;MsgBox(4096,"Len of $aCurrency[1]",StringLen($aCurrency[1])&"  -  "&$aCurrency[1])
	If StringLen(StringStripWS($aCurrency[1],3))<=$nWidth Then
		SetError(1)
		Return $sStrippedCurrency
	EndIf
	; Valid string - Check for Currency Symbol
	If $sHasCSymbol = 0 Then  ; No Symbol
		If $nCSymbolSize > 0 Then  ; Parameter error
			SetError(1)
			Return $sStrippedCurrency
		EndIf
		$sSymbol = ""	; no symbol
		$sDelValue = $aCurrency[1]
	Else
		If $sHasCSymbol = 1 Then
			$sSymbol = StringLeft($aCurrency[1],$nCSymbolSize)	; prefix
			$sDelValue = StringMid($aCurrency[1], $nCSymbolSize+1)
		Else  ;suffix
			If $aCurrency[0]>1 Then ; is a decimal point
				$sSymbol = StringRight($aCurrency[2],$nCSymbolSize)	; suffix
				$sDelValue = $aCurrency[1]
			EndIf
		EndIf
	EndIf
	; create the new string - include delimters
	$nLength = StringLen($aCurrency[1])
	$nNoDel = _Floor( $nLength / $nWidth)
	$nPreDel = $nLength - ($nNoDel * $nWidth)
	$sDelStr = ""
	If $nPreDel > 0 Then
		$sDelStr = StringLeft($sDelValue,$nPreDel) ; First portion of String + Delimiter char
	EndIf
	$nCnt = $nPreDel + 1	; start with number folllowing $nPreDel eg 1 + 1

;	MsgBox(4096, "Format", $nLength & @CRLF & $nNoDel & @CRLF & $nPreDel & @CRLF & $sDelStr & @CRLF & $sDelValue)

	For $nLoop = 0 To $nNoDel - 1
		If StringLen($sDelStr) = 0 Then
			$sDelStr &= StringMid($sDelValue, $nCnt, $nWidth)
		Else
			$sDelStr &= $cChar & StringMid($sDelValue, $nCnt, $nWidth)
		EndIf

		$nCnt = $nCnt + $nWidth
	Next
	; set up $sNewCurrency
	$sNewCurrency = ""
	If $aCurrency[0] = 1 Then
		If $sHasCSymbol = 0 Then
			$sNewCurrency = $sDelStr
		Else
			If $sHasCSymbol = 1 Then ; prefix
				$sNewCurrency = $sSymbol & $sDelStr
			Else
				$sNewCurrency = $sDelStr & $sSymbol ; suffix
			EndIf
		EndIf
	Else ; there are decimal places
		If $sHasCSymbol = 0 Then
			$sNewCurrency = $sDelStr & $cDecimalChar & $aCurrency[2]
		Else
			If $sHasCSymbol = 1 Then ; prefix
				$sNewCurrency = $sSymbol & $sDelStr & $cDecimalChar & $aCurrency[2]
			Else
				$sNewCurrency = $sDelStr & $cDecimalChar & $aCurrency[2] & $sSymbol ; suffix
			EndIf
		EndIf
	EndIf

	Return $sNewCurrency
EndFunc   ;==>_StringCurencyFormat


;~ Func SQLiteCreateDBandTables()
;~ 	
;~ Local $hQuery, $aRow, $sMsg
;~ 	_SQLite_Startup ()
;~ 	_SQLite_Open () ; open :memory: Database
;~ 	_SQLite_QueryNoResult (-1, "CREATE TABLE aTest (a,b,c);") ; CREATE a Table
;~ 	_SQLite_QueryNoResult (-1, "INSERT INTO aTest(a,b,c) VALUES ('c','2','World');") ; INSERT Data
;~ 	_SQLite_QueryNoResult (-1, "INSERT INTO aTest(a,b,c) VALUES ('b','3',' ');") ; INSERT Data
;~ 	_SQLite_QueryNoResult (-1, "INSERT INTO aTest(a,b,c) VALUES ('a','1','Hello');") ; INSERT Data
;~ 	_SQlite_Query (-1, "SELECT c FROM aTest ORDER BY a;", $hQuery) ; the query
;~ 	While _SQLite_FetchData ($hQuery, $aRow) = $SQLITE_OK
;~ 		$sMsg &= $aRow[0]
;~ 	WEnd
;~ 	_SQLite_QueryNoResult (-1, "DROP TABLE aTest;") ; Remove the table
;~ 	ConsoleWrite($sMsg & @CR)
;~ 	_SQLite_Close()
;~ 	_SQLite_Shutdown()
;~ EndFunc
